////////////////////////////////////////////////////////////////////////////////
// MAIN.C - FPStest: A wireframe game engine by SSJX   ported by Athlor
////////////////////////////////////////////////////////////////////////////////
#include <pager.h>
#include <ribbon.h>
#include "icon.h"

char VersionPtr[] = "FPStest";  // Program name (Ribbon name)
int AppStackSize = 3000;

struct { // Object and Point Structs
 long x;
 long y;
} pnt[20];

const int floorobj[] = {
 -32, 32, 32, 32, 32, 32, 32, 32, -32, -32, 32, -32
};

const int floor[] = { 0, 1, 1, 2, 2, 3, 3, 0 };

const int wallobj[] = {
 0, -32, 32, 0, 32, 32, 0, 32, -32, 0, -32, -32
};

const int wallobj2[] = {
 32, -32, 0, 32, 32, 0, -32, 32, 0, -32, -32, 0
};

const int wall[] = { 0, 1, 1, 2, 2, 3, 3, 0 };

const int pyrobj[] = { // NOTE - pyramid is not filled in
 -32, 32, 32, 32, 32, 32, 32, 32, -32, -32, 32, -32, 0, 0, 0
};

const int pyr[] = { 0, 1, 1, 2, 2, 3, 3, 0, 0, 4, 1, 4, 2, 4, 3, 4 };

const char maze[28][28] = { // 0=Floor (not shown)  1,2 = Wall  3=little pyramid
 {0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0},
 {2, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2},
 {2, 0, 3, 0, 0, 0, 0, 0, 2, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 3, 0, 2},
 {2, 0, 0, 0, 0, 0, 0, 0, 2, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2},
 {2, 1, 0, 1, 1, 2, 0, 0, 2, 1, 1, 0, 0, 0, 2, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 1, 2},
 {2, 0, 0, 0, 0, 2, 0, 0, 2, 0, 0, 0, 0, 0, 2, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2},
 {2, 0, 0, 0, 0, 2, 0, 0, 2, 0, 0, 0, 0, 0, 2, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2},
 {2, 0, 0, 0, 0, 2, 0, 0, 2, 0, 0, 0, 0, 0, 2, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2},
 {2, 0, 0, 0, 0, 2, 0, 0, 2, 0, 0, 0, 0, 0, 2, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2},
 {2, 0, 0, 0, 0, 2, 0, 0, 2, 0, 0, 0, 0, 0, 2, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2},
 {2, 0, 0, 1, 1, 2, 0, 0, 2, 1, 1, 0, 0, 0, 2, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2},
 {2, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2},
 {2, 0, 0, 0, 0, 3, 0, 0, 3, 0, 0, 0, 0, 0, 2, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2},
 {2, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2, 0, 0, 3, 0, 0, 0, 3, 0, 0, 0, 0, 0, 2},
 {2, 1, 1, 1, 1, 2, 4, 0, 2, 1, 1, 1, 1, 0, 2, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2},
 {2, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2, 0, 0, 0, 0, 4, 0, 0, 0, 0, 0, 0, 0, 2},
 {2, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2},
 {2, 0, 0, 1, 1, 2, 0, 0, 2, 1, 1, 0, 0, 0, 2, 0, 0, 3, 0, 0, 0, 3, 0, 0, 0, 0, 0, 2},
 {2, 0, 0, 0, 0, 2, 0, 0, 2, 0, 0, 0, 0, 0, 2, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2},
 {2, 0, 0, 0, 0, 2, 0, 0, 2, 0, 0, 0, 0, 0, 2, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2},
 {2, 0, 0, 0, 0, 2, 0, 0, 2, 0, 0, 0, 0, 0, 2, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2},
 {2, 0, 0, 0, 0, 2, 0, 0, 2, 0, 0, 0, 0, 0, 2, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2},
 {2, 0, 0, 0, 0, 2, 0, 0, 2, 0, 0, 0, 0, 0, 2, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2},
 {2, 1, 1, 1, 1, 2, 0, 0, 2, 0, 0, 0, 0, 0, 2, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 2},
 {2, 0, 0, 0, 0, 0, 0, 0, 2, 1, 1, 0, 0, 0, 2, 0, 0, 0, 0, 0, 0, 0, 0, 2, 0, 0, 0, 2},
 {2, 0, 3, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 3, 0, 2},
 {2, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2},
 {0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0},
};

struct { // Main object list
 long x;
 long z;
 char no;
} object[1000];

struct { // Order depth list (For polygon ordering) 
 int no;
 long dist;
} objdepth[300];
/*
struct {
 int no;
 long dist;
} tmpdepth;*/

struct {
 long x;
 long z;
 long rx;
 long rz;
} player;

struct edge { // Edge struct - used to store the
 char act;    // left and right values for poly filling
 int min;     // USED BY MYLINE FUNCTION
 int max;
};

struct edge fill[2]; // Was 120

const char Cos[360] = {
 127, 127, 127, 127, 127, 127, 127, 127, 126, 126,
 126, 125, 125, 124, 124, 123, 123, 122, 121, 121,
 120, 119, 118, 117, 116, 116, 115, 114, 113, 111,
 110, 109, 108, 107, 106, 104, 103, 102, 100, 99,
 98, 96, 95, 93, 92, 90, 88, 87, 85, 83,
 82, 80, 78, 77, 75, 73, 71, 69, 67, 65,
 64, 62, 60, 58, 56, 54, 52, 50, 47, 45,
 43, 41, 39, 37, 35, 33, 30, 28, 26, 24,
 22, 20, 17, 15, 13, 11, 8, 6, 4, 2,
 0, -2, -4, -6, -8, -11, -13, -15, -17, -20,
 -22, -24, -26, -28, -30, -33, -35, -37, -39, -41,
 -43, -45, -47, -50, -52, -54, -56, -58, -60, -62,
 -63, -65, -67, -69, -71, -73, -75, -77, -78, -80,
 -82, -83, -85, -87, -88, -90, -92, -93, -95, -96,
 -98, -99, -100, -102, -103, -104, -106, -107, -108, -109,
 -110, -111, -113, -114, -115, -116, -116, -117, -118, -119,
 -120, -121, -121, -122, -123, -123, -124, -124, -125, -125,
 -126, -126, -126, -127, -127, -127, -127, -127, -127, -127,
 -127, -127, -127, -127, -127, -127, -127, -127, -126, -126,
 -126, -125, -125, -124, -124, -123, -123, -122, -121, -121,
 -120, -119, -118, -117, -116, -116, -115, -114, -113, -111,
 -110, -109, -108, -107, -106, -104, -103, -102, -100, -99,
 -98, -96, -95, -93, -92, -90, -88, -87, -85, -83,
 -82, -80, -78, -77, -75, -73, -71, -69, -67, -65,
 -64, -62, -60, -58, -56, -54, -52, -50, -47, -45,
 -43, -41, -39, -37, -35, -33, -30, -28, -26, -24,
 -22, -20, -17, -15, -13, -11, -8, -6, -4, -2,
 0, 2, 4, 6, 8, 11, 13, 15, 17, 20,
 22, 24, 26, 28, 30, 33, 35, 37, 39, 41,
 43, 45, 47, 50, 52, 54, 56, 58, 60, 62,
 63, 65, 67, 69, 71, 73, 75, 77, 78, 80,
 82, 83, 85, 87, 88, 90, 92, 93, 95, 96,
 98, 99, 100, 102, 103, 104, 106, 107, 108, 109,
 110, 111, 113, 114, 115, 116, 116, 117, 118, 119,
 120, 121, 121, 122, 123, 123, 124, 124, 125, 125,
 126, 126, 126, 127, 127, 127, 127, 127, 127, 127,
};

const char Sin[360] = {
 0, 2, 4, 6, 8, 11, 13, 15, 17, 20,
 22, 24, 26, 28, 30, 33, 35, 37, 39, 41,
 43, 45, 47, 50, 52, 54, 56, 58, 60, 62,
 63, 65, 67, 69, 71, 73, 75, 77, 78, 80,
 82, 83, 85, 87, 88, 90, 92, 93, 95, 96,
 98, 99, 100, 102, 103, 104, 106, 107, 108, 109,
 110, 111, 113, 114, 115, 116, 116, 117, 118, 119,
 120, 121, 121, 122, 123, 123, 124, 124, 125, 125,
 126, 126, 126, 127, 127, 127, 127, 127, 127, 127,
 127, 127, 127, 127, 127, 127, 127, 127, 126, 126,
 126, 125, 125, 124, 124, 123, 123, 122, 121, 121,
 120, 119, 118, 117, 116, 116, 115, 114, 113, 111,
 110, 109, 108, 107, 106, 104, 103, 102, 100, 99,
 98, 96, 95, 93, 92, 90, 88, 87, 85, 83,
 82, 80, 78, 77, 75, 73, 71, 69, 67, 65,
 64, 62, 60, 58, 56, 54, 52, 50, 47, 45,
 43, 41, 39, 37, 35, 33, 30, 28, 26, 24,
 22, 20, 17, 15, 13, 11, 8, 6, 4, 2,
 0, -2, -4, -6, -8, -11, -13, -15, -17, -20,
 -22, -24, -26, -28, -30, -33, -35, -37, -39, -41,
 -43, -45, -47, -50, -52, -54, -56, -58, -60, -62,
 -63, -65, -67, -69, -71, -73, -75, -77, -78, -80,
 -82, -83, -85, -87, -88, -90, -92, -93, -95, -96,
 -98, -99, -100, -102, -103, -104, -106, -107, -108, -109,
 -110, -111, -113, -114, -115, -116, -116, -117, -118, -119,
 -120, -121, -121, -122, -123, -123, -124, -124, -125, -125,
 -126, -126, -126, -127, -127, -127, -127, -127, -127, -127,
 -127, -127, -127, -127, -127, -127, -127, -127, -126, -126,
 -126, -125, -125, -124, -124, -123, -123, -122, -121, -121,
 -120, -119, -118, -117, -116, -116, -115, -114, -113, -111,
 -110, -109, -108, -107, -106, -104, -103, -102, -100, -99,
 -98, -96, -95, -93, -92, -90, -88, -87, -85, -83,
 -82, -80, -78, -77, -75, -73, -71, -69, -67, -65,
 -64, -62, -60, -58, -56, -54, -52, -50, -47, -45,
 -43, -41, -39, -37, -35, -33, -30, -28, -26, -24,
 -22, -20, -17, -15, -13, -11, -8, -6, -4, -2,
};

// Projection functions:
// tdpoint  - Quick 3d>2d Just used to see if the poly is visible
// tdpoint2 - Main 3d>2d function
// rot      - 2d point rotation, used for the map

void tdpoint(long *ox, long *oy, long *oz, long x, long y, long z, int sd, int cd)
{
  long tx, tz;
  tz = ((z*cd)) + ((x*sd)) >> 8; // Divide by 256 
  if (tz>0 & tz<600) // The next two numbers set the view depth
  {
    tx = ((x*cd)) - ((z*sd)) >> 1; // Divide by 2
    *oy = 80; // was 50
    *ox = 80 + (tx/tz);
    *oz = tz;
  }
  else
  {
    *ox = -1;
  }
}

void tdpoint2(long *ox, long *oy, long x, long y, long z, int sd, int cd)
{
  long tx, tz;
  tx = ((x*cd)) - ((z*sd)) >> 1; // Divide by 2
  tz = ((z*cd)) + ((x*sd)) >> 8; // Divide by 256
  *oy = 80 + ((y << 7) / tz); // 80 was 50
  *ox = 80 + (tx / tz);
}

void rot(long *rx, long *ry, long x, long y, int sd, int cd)
{
  *rx = ((x*cd) - (y*sd)) >> 8; // Divide by 256
  *ry = ((x*sd) + (y*cd)) >> 8; // Divide by 256
}

// myline fills the edge fill array (?) with left and right co-ordindates
// so that the actual fill routine just needs to do a hline between the two

void myline(struct edge *lr, int x1, int y1, int x2, int y2)
{
  int i, xx, c;
  if (y2 > y1)
  {
    for (i=0; i<(y2-y1); i++)
    {
      c = y1 + i;
      if (c >= 0 && c<100)
      {
        xx = x1 + (i * (x2-x1)) / (y2-y1);
        lr[c].act = 1;
        if(xx<lr[c].min) lr[c].min = xx;
        if(xx>lr[c].min) lr[c].min = xx;
      }
    }
  }
  if (y2 < y1)
  {
    for (i=0; i<(y1-y2); i++)
    {
      c = y1 - i;
      if (c >= 0 && c<100)
      {
        xx = x1 - (i * (x2 - x1)) / (y2 - y1);
        lr[c].act = 1;
        if(xx<lr[c].min) lr[c].min = xx;
        if(xx>lr[c].min) lr[c].min = xx;
      }
    }
  }
  if (y1 == y2)
  {
    if (y1 >= 0 && y1<100)
    {
      if(x1<lr[y1].min) lr[y1].min = x1;
      if(x2<lr[y1].min) lr[y1].min = x2;
      if(x1>lr[y1].min) lr[y1].min = x1;
      if(x2>lr[y1].min) lr[y1].min = x2;
    }
  }
}

////////////////////////////////////////////////////////////////////////////////
void FPSThread(void)
{
  MESSAGE msg;
  long rx, ry, rz, i, c, x, z, poly_x, poly_z;
  int pl, polymax, maxobs, sd, cd, cy, ang = 0;
  PID pid;
  pid.Name = VersionPtr;
  pid.EnableForeground = TRUE;
  pid.Icon = NULL;
  RimSetPID(&pid);

  player.x = 0; // Player start positions
  player.z = 0;
  player.rx = player.x;
  player.rz = player.z;

  c = 0;  // Create object list
/*
  for(z=0;z<28;z++) // Floor grid
  {
  	for(x=0;x<28;x++)
  	{
  	object[c].no=0;
  	object[c].x=(x-7)*64;
  	object[c].z=(z-7)*64;
  	c++;
  	}
  }*/
  for (z=0; z<28; z++)  // Everything else
  {
    for (x=0; x<28; x++)
    {
      if (maze[x][z] != 0)
      {
        object[c].no = maze[x][z];
        object[c].x = (x-7)*64;
        object[c].z = (z-7)*64;
        c++;
      }
    }
  }
  maxobs = c;
/*
  for (i = 0; i < 110; i++) // Clear fill array
  {
    fill[i].act = 0;
    fill[i].max = 0;
    fill[i].min = 9999;
  }*/
  sd = Sin[ang];
  cd = Cos[ang];

  for (;;)  ///////////////////////////////// Start main loop
  {
      LcdClearDisplay();
      c = 0;  // Find out what objects we can see and add them to the list
      for (cy=0; cy<maxobs; cy++)
      {
        poly_z = object[cy].z - player.z;
        poly_x = object[cy].x - player.x;
        tdpoint(&rx, &ry, &rz, poly_x, 0, poly_z, sd, cd);
        if (rx>0 && rx<160)
        {
          objdepth[c].no = cy;
          objdepth[c].dist = rz;
          c++;
        }
      }
      polymax = c;
      for (pl=0; pl<polymax; pl++) // Show sorted polygons
      {
        cy = objdepth[pl].no;
        poly_z = object[cy].z - player.z;
        poly_x = object[cy].x - player.x;
        c = 0;

        switch (object[cy].no)  // Start of objects
        {
          case 0:  // Open space
            for (i=0; i<4; i++) // Create points list 3d->2d (4 points)
            {
              rx = floorobj[c++] + poly_x;
              ry = floorobj[c++];
              rz = floorobj[c++] + poly_z;
              tdpoint2(&pnt[i].x, &pnt[i].y, rx, ry, rz, sd, cd);
            }
            c = 0;
            for (i=0; i<4; i++) // Draw 3d lines (4 lines)
            {
              LcdDrawLine(DRAW_BLACK,pnt[floor[c]].x,pnt[floor[c]].y,pnt[floor[c+1]].x,pnt[floor[c+1]].y);
              c += 2;
            }
            break;
          case 1:  // Wall 1
            for (i=0; i<4; i++) // Create points list 3d->2d (4 points)
            {
              rx = wallobj[c++] + poly_x;
              ry = wallobj[c++];
              rz = wallobj[c++] + poly_z;
              tdpoint2(&pnt[i].x, &pnt[i].y, rx, ry, rz, sd, cd);
            }
            c = 0;
            for (i=0; i<4; i++) // Draw 3d lines (4 lines)
            {
              LcdDrawLine(DRAW_BLACK,pnt[wall[c]].x, pnt[wall[c]].y, pnt[wall[c+1]].x, pnt[wall[c+1]].y);
              c += 2;
            }
            break;
          case 2:  // Wall 2 
            for (i=0; i<4; i++) // Create points list 3d->2d
            {
              rx = wallobj2[c++] + poly_x;
              ry = wallobj2[c++];
              rz = wallobj2[c++] + poly_z;
              tdpoint2(&pnt[i].x, &pnt[i].y, rx, ry, rz, sd, cd);
            }
            c = 0;
            for (i=0; i<4; i++) // Draw 3d lines 
            {
              LcdDrawLine(DRAW_BLACK,pnt[wall[c]].x, pnt[wall[c]].y, pnt[wall[c+1]].x, pnt[wall[c+1]].y);
              c += 2;
            }
            break;
          case 3:  // Pyramid 
            for (i=0; i<5; i++) // Create points list 3d->2d
            {
              rx = pyrobj[c++] + poly_x;
              ry = pyrobj[c++];
              rz = pyrobj[c++] + poly_z;
              tdpoint2(&pnt[i].x, &pnt[i].y, rx, ry, rz, sd, cd);
            }
            c = 0;
            for (i=0; i<8; i++) // Draw 3d lines 
            {
              LcdDrawLine(DRAW_BLACK,pnt[pyr[c]].x, pnt[pyr[c]].y, pnt[pyr[c+1]].x, pnt[pyr[c + 1]].y);
              c += 2;
            }
            break;
        }
      }

    RimGetMessage(&msg);
    switch (msg.Device)
    {
      case DEVICE_KEYPAD:
        switch (msg.Event)
        {
          case THUMB_ROLL_DOWN:
            ang += 10;
            if (ang>359) ang -= 360;
            sd = Sin[ang];
            cd = Cos[ang];
            break;
          case THUMB_ROLL_UP:
            ang -= 10;
            if (ang<0) ang += 360;
            sd = Sin[ang];
            cd = Cos[ang];
            break;
          case KEY_DOWN: case KEY_REPEAT: // KEY_REPEAT if enabled crashes unless
          {                               // Myline function is UNREMARKED - HOW ODD !
            switch (msg.SubMsg)
            {
              case KEY_ESCAPE:  // ESCAPE key exits
                RibbonShowRibbon();
                RimTerminateThread();
                break;
              case 'a':
                player.rx += (sd << 3); // Multiply by 8
                player.rz += (cd << 3); // Multiply by 8
                player.x = player.rx >> 8; // Divide by 256
                player.z = player.rz >> 8; // Divide by 256
                break;
              case 'z':
                player.rx -= (sd << 3); // Multiply by 8
                player.rz -= (cd << 3); // Multiply by 8
                player.x = player.rx >> 8; // Divide by 256
                player.z = player.rz >> 8; // Divide by 256
                break;
            }
            break;
          }
        }
    } // msg.device
  } // Main loop
}

void PagerMain(void)
{
  RibbonRegisterFunction(VersionPtr, &IconBMP, 0, 40, FPSThread, AppStackSize);
}
