////////////////////////////////////////////////////////////////////////////////
// Gravity: Ported from Amiga Amos program by Paithan/Depth (ptn@c2i.net)
////////////////////////////////////////////////////////////////////////////////
#include <pager.h>
#include <ribbon.h>
#include "icon.h"

char VersionPtr[] = "Gravity"; // Program name (Ribbon name)
int AppStackSize = 3000;

int seed;

int rnd(int Value)
{
  seed *= 20077;
  seed += 12345;
 return ((((seed>>16)&32767)*Value)>>15);
}

int sqrt(unsigned long a)
{
  int rem=0,root=0,divisor=0,i;
  for(i=0; i<16; i++)
  {
    root <<= 1;
    rem = ((rem<< 2) + (a >> 30));
    a <<= 2;
    divisor = (root<<1) + 1;
    if(divisor <= rem){
      rem -= divisor;
      root++;
    }
  }
  return (root);
}

void GravityThread(void)
{
  int OrbitX, OrbitY, CenterX, CenterY, Gravity, N, L, Num = 50;
  int XDIF, YDIF, PX[50], PY[50], VX[50], VY[50];
  char buf[64];
  MESSAGE msg;
  PID pid;
  pid.Name = VersionPtr;
  pid.EnableForeground = TRUE;
  pid.Icon = NULL;
  RimSetPID(&pid);
  seed = RimGetTicks(); // Seed Rnd function
  RimRequestForeground(RimGetCurrentTaskID());

  for(N=1; N<51; N++) // FOR N = 1 TO Num
  {
    PX[N] = rnd(24) + 24; // 32
    PY[N] = rnd(15) + 15; // 20
    VX[N] = rnd(8) - 4; // 10 -5
    VY[N] = rnd(8) - 4; // 10 -5
  } // NEXT N
  OrbitX = 20; // 20
  OrbitY = 20; // 20
  CenterX = 60; // Screen width
  CenterY = 60; // Screen Height
  Gravity = 1;

  for (;;) // Start message loop
  {
    while(RimPeekMessage()==FALSE)
    {
      for(N=1; N<Num; N++) // FOR N = 1 TO Num - 1
      {
        LcdDrawBox(DRAW_WHITE,PX[N]+CenterX,PY[N]+CenterY,PX[N]+CenterX+2,PY[N]+CenterY+2);
        XDIF = OrbitX - PX[N];
        YDIF = OrbitY - PY[N];
        //L = sqrt(XDIF*XDIF+YDIF*YDIF);
        L = sqrt(XDIF * XDIF + YDIF * YDIF);
        L*=Gravity; // L = L*Gravity;
        if(L>0) // IF L > 0 THEN
        {
          VX[N] = VX[N]+XDIF/L;
          VY[N] = VY[N]+YDIF/L;
          PX[N] = PX[N]+VX[N];
          PY[N] = PY[N]+VY[N];
        } // END IF
        LcdDrawBox(DRAW_BLACK,PX[N]+CenterX,PY[N]+CenterY,PX[N]+CenterX+2,PY[N]+CenterY+2);
      } // NEXT N
      RimSprintf(buf,63,"GRAVITY: %d  ",Gravity);
      LcdPutStringXY(0,152,buf,-1,0);
      RimTaskYield();
      RimSleep(5);
    }
    RimGetMessage(&msg);
    switch (msg.Device)
    {
      case DEVICE_KEYPAD:
        switch (msg.Event)
        {
          case THUMB_ROLL_UP:
            Gravity++;
            if(Gravity>9) Gravity=9;
            break;
          case THUMB_ROLL_DOWN:
            Gravity--;
            if(Gravity< 0) Gravity= 0;
            break;
          case KEY_DOWN:
          {
            switch (msg.SubMsg)
            {
              case KEY_ESCAPE: // ESC key exits
                RibbonShowRibbon();
                RimTerminateThread();
                break;
            }
            break;
          }
        }
    }
  } // Message loop
}

void PagerMain(void)
{
  RibbonRegisterFunction(VersionPtr, &IconBMP, 0, 40, GravityThread, AppStackSize);
}
