////////////////////////////////////////////////////////////////////////////////
// MAIN.C - Life: A 2-d cellular automaton example for 857 & 957
////////////////////////////////////////////////////////////////////////////////
#include <pager.h>
#include <ribbon.h>
#include "icon.h"
#define ROWS 40 //160  // Screen height
#define COLS 40 //160  // Screen width

char VersionPtr[] = "Life"; // Program name (Ribbon name)
int AppStackSize = 3000;

int civ1[ROWS][COLS], civ2[ROWS][COLS], seed;

int rand(int Value)
{
  seed *= 20077;
  seed += 12345;
 return ((((seed>>16)&32767)*Value)>>15);
}

void update_generation(int old[ROWS][COLS], int new[ROWS][COLS])
{
  int i, j, count;
  for(i=0; i<ROWS; ++i)
  {
    for(j=0; j<COLS; ++j)
    {
      count = old[(i+ROWS-1)%ROWS][(j+COLS-1)%COLS]+old[(i+ROWS-1)%ROWS][j]+old[(i+ROWS-1)% ROWS][(j+1)%COLS]+old[i][(j+COLS-1)%COLS]+old[i][(j+1)%COLS]+old[(i+1)%ROWS][(j+COLS-1)%COLS]+old[(i+1)%ROWS][j]+old[(i+1)% ROWS][(j+1)%COLS];
      switch (count)
      {
        case 0: case 1: case 4: case 5: case 6: case 7: case 8:
          new[i][j] = 0;
          break;
        case 2:
          new[i][j] = old[i][j];
          break;
        case 3:
          new[i][j] = 1;
          break;
      }
//    LcdSetPixel(i*4+1,j*4+1,new[i][j] ? TRUE : FALSE);
      LcdDrawBox(new[i][j] ? TRUE : FALSE,(i<<2)+1,(j<<2)+1,(i<<2)+3,(j<<2)+3);
    }
  }
}

void initialize(void)
{
  int i, j;
  for(i=0; i<ROWS; ++i)
  {
    for(j=0; j<COLS; ++j)
    {
      civ1[i][j] = rand(2);
//    LcdSetPixel(i*4+1,j*4+1,civ1[i][j] ? TRUE : FALSE);
      LcdDrawBox(civ1[i][j] ? TRUE : FALSE,(i<<2)+1,(j<<2)+1,(i<<2)+3,(j<<2)+3);
    }
  }
}

void LifeThread(void)
{
  MESSAGE msg;
  PID pid;
  pid.Name = VersionPtr;
  pid.EnableForeground = TRUE;
  pid.Icon = NULL;
  RimSetPID(&pid);
  seed = RimGetTicks(); // Seed Rnd function
  RimRequestForeground(RimGetCurrentTaskID());
  initialize();

  for(;;) // Start message loop
  {
    while(RimPeekMessage()==FALSE)
    {
      update_generation(civ1, civ2);
      update_generation(civ2, civ1);
      RimTaskYield();
    }
    RimGetMessage(&msg);
    if(msg.Device==DEVICE_KEYPAD&&msg.Event==KEY_DOWN)
    {
      if(msg.SubMsg==KEY_ESCAPE) // ESC quits
      {
        RibbonShowRibbon();
        RimTerminateThread();
      }
    }
  } // Message loop
}

void PagerMain(void)
{
  RibbonRegisterFunction(VersionPtr, &IconBMP, 0, 40, LifeThread, AppStackSize);
  return;
}
