/*-------------------------------------------------------------------------*
 *
 * RIM sample application:
 * BOUNCER.C
 *
 * This program demonstrates the use of multiple threads and timers,
 * and multiple threads writing to the same display context.
 *
 * Copyright 1997-2000 Research In Motion Limited.  All Rights Reserved
 *-------------------------------------------------------------------------*/
#include "Pager.h"
#include "led.h"

char VersionPtr[] = "Bouncer";// Declare the name of this application
int AppStackSize = 1000; // Delcare the stack space requirements

// Declare static variables, which are used to communicate initialization
// data to the threads. 

static int OriginalContext;
static int Period;
static int Letter;
static int ovx, ovy;

static LcdConfig LcdConf;

// Prototype for thread entry point of threads to be created.
void BounceThread(void);

// TRYED
// -subsystem:windows -machine:ix86 -DLL /EXPORT:RimHeader OSEntry.obj rimos.lib
// -subsystem:windows -machine:ix86 -DLL /DEF:bouncer.def OSEntry.obj rimos.lib

//-----------------------------------------------------------------------
// This function is the main entry point for applications.
//-----------------------------------------------------------------------
void PagerMain(void)
{
  int t;
  MESSAGE msg;
  MESSAGE sendmsg;

#ifdef LED_MESSAGE
  static BOOL led;
#endif

  // Hold the thread IDs as created.
  TASK ThreadNos[4];

  LcdGetConfig(&LcdConf);
  RimRequestForeground(RimGetCurrentTaskID()); // Temp added to test

  LcdPutStringXY(0, 0, "BOUNCER.C\n" "Stop bounce with a-d\n" "Restart with A-D\n", -1, LCD_MULTILINE);

  // Save the current display context, so the new threads can use it.
  OriginalContext = LcdGetDisplayContext();

  // Start the threads, telling each one through static variables what it is to do.
  for (t = 0; t < 4; t++)
  {
    Period = 5 + t * 2;
    Letter = 'a' + t;
    ovx = 1 + t;
    ovy = 4 - t;

    // Create the thread, and save the task ID.
    ThreadNos[t] = RimCreateThread(BounceThread, 1000);

    // The thread will start to run until it yields on RimTaskYield().
    // This allows the new thread to copy the static variables.
    RimTaskYield();
  }

  // Start the message loop for this program.
  for (;;)
  {
    int NewContrast;

    RimGetMessage(&msg);
    switch (msg.Device)
    {
      case DEVICE_HOLSTER:
        // On holster events, steal the foreground in bouncer
        // (bouncer is a bit obnoxious)
        RimRequestForeground(RimGetCurrentTaskID());
        break;

      case DEVICE_KEYPAD:
        switch (msg.Event)
        {
          case KEY_DOWN:
            switch (msg.SubMsg)
            {

              case 'a':
              case 'b':
              case 'c':
              case 'd':
                // Pushing a lower case letter sends a message to the thread
                // telling it to stop bouncing.
                sendmsg.Device = 100;
                RimPostMessage(ThreadNos[msg.SubMsg - 'a'], &sendmsg);
                break;
              case 'A':
              case 'B':
              case 'C':
              case 'D':
                // Pushing an upper case letter sends a message to the thread
                // telling it to start bouncing again.
                sendmsg.Device = 101;
                RimPostMessage(ThreadNos[msg.SubMsg - 'A'], &sendmsg);
                break;
              case 's':
                // Do sound alert 1.
                LcdClearDisplay();
                LcdPutStringXY(20, 20, " Sound alert 1! ", -1, 0);
                RimAlertNotify(TONE_SEQUENCE_1, -1);
                break;
              case 'S':
                // Do sound alert 2.
                LcdClearDisplay();
                LcdPutStringXY(20, 20, " Sound alert 2! ", -1, 0);
                RimAlertNotify(TONE_SEQUENCE_2, -1);
                break;
#ifdef LED_MESSAGE
              case 'l':
                // Turn LED on or off
                led = (led == LED_OFF) ? LED_ON : LED_OFF;
                RimSetLed(LED_MESSAGE, led);
                break;
              case 'L':
                // Turn LED blinking on or off
                led = (led == LED_BLINK) ? LED_ON : LED_BLINK;
                RimSetLed(LED_MESSAGE, led);
                break;
#endif
            }
            break;
          case THUMB_CLICK:
            // On roller presses, clear all the junk off the display.
            LcdClearDisplay();
          // Fall through to redraw the bar.
          case THUMB_ROLL_UP:
          case THUMB_ROLL_DOWN:
            // Do a very simple contrast bargraph on roller rolls.

            NewContrast = LcdGetContrast();

            // Adjust contrast according to roller delta.
            if (msg.Event == THUMB_ROLL_UP)
            {
              NewContrast += (int)msg.SubMsg;
            }
            else
            {
              NewContrast -= (int)msg.SubMsg;
            }
            // Limit contrast range.
            if (NewContrast > LCD_MAXCONTRAST)
              NewContrast = LCD_MAXCONTRAST;
            if (NewContrast < 0)
              NewContrast = 0;
            // Set the contrast
            LcdSetContrast(NewContrast);
            // Show a simple bargraph on the right.
            LcdDrawLine(DRAW_WHITE, LcdConf.width - 2, LcdConf.height - 1, LcdConf.width - 2, 0);
            LcdDrawLine(DRAW_BLACK, LcdConf.width - 2, LcdConf.height - 1, LcdConf.width - 2, LcdConf.height - (NewContrast * LcdConf.height) / LCD_MAXCONTRAST);
            break;
        }
    }

  }
}

//-----------------------------------------------------------------------
// This function implements the message loop for a thread to bounce
// a letter around the screen.  Four threads are created using the same
// function.  Note that local variables are local to each thread, while
// static variables are shared between instances.
//-----------------------------------------------------------------------
void BounceThread(void)
{
  MESSAGE msg;
  int x, y;
  int vx, vy;
  int ThisLetter;
  int ThisPeriod;

  // The RimCreateThread() call will cause the new thread to run as soon
  // as the creating task does a RimTaskYield().  The new thread in turn
  // runs until it does a RimGetMessage() or RimTaskYield().
  // This program uses this sequence of events to pass data to each
  // thread using static variables.

  // Thus, the static variables will hold the values they contained before
  // RimCreateThread() was called until the first RimGetMessage() 
  // or RimTaskYield() is called

  ThisPeriod = Period;
  x = 0;
  y = 0;
  vx = ovx;
  vy = ovy;
  ThisLetter = Letter;

  // Reset the initial x and y position
  x = 0;
  y = 20;

  // Set a periodic timer for bouncing a letter around the display.
  RimSetTimer(0, Period, TIMER_PERIODIC);

  // Each thread by default gets its own display context.  In order
  // to have all four threads use the same display context, it is
  // necessary to manually set it back to the original context.
  // This call is necessary here because all threads write to the same
  // LCD display context in this program.
  LcdSetDisplayContext(OriginalContext);

  // The message loop.
  for (;;)
  {
    RimGetMessage(&msg);

    switch (msg.Device)
    {
      case DEVICE_TIMER:
        // Its a timer message.
        break;
      case 100:
        // Its a message from the main thread saying to stop bouncing.
        RimKillTimer(0);
        break;
      case 101:
        // Its a message from the main thread to restart the bouncing.
        RimSetTimer(0, ThisPeriod, TIMER_PERIODIC);
        break;
    }
    // Add the velocity to the position
    x += vx;
    y += vy;

    // Handle any collisions with the walls.
    if (x < 0)
    {
      x = -x;
      vx = -vx;
    }
    if (y < 0)
    {
      y = -y;
      vy = -vy;
    }
    if (x > LcdConf.width - 10)
    {
      x = -x + (LcdConf.width - 10) * 2;
      vx = -vx;
    }
    if (y > LcdConf.height - 10)
    {
      y = -y + (LcdConf.height - 10) * 2;
      vy = -vy;
    }
    LcdPutStringXY(x, y, (char *)&ThisLetter, 1, 0); // Draw the letter
  }
}
