/*-------------------------------------------------------------------------*
 *
 * RIM sample application:
 * COMTEST.C
 *
 * This program demonstrates the use of the COM port API.
 *
 * Copyright 1997-2000 Research In Motion Limited.  All Rights Reserved
 *-------------------------------------------------------------------------*/
#include "Pager.h"
#include <string.h>

char VersionPtr[] = "COM Test";
int AppStackSize = 2000;

void PagerMain(void)
{
  MESSAGE msg;
  char ch;
  int x;

  int RxIndex;
  char RxBuffer[17];

  int TxIndex;
  char TxBuffer[17];

  char *StatusLine;

  RxIndex = 0;
  TxIndex = 0;

  // Test out the notify pattern feature - when we see 'abcd' on a closed
  // serial port, this program will get notified.
  CommRegisterNotifyPattern("abcd");

  StatusLine = "Push 'O' to open port";

  for (;;)
  {
    // Display received characters
    LcdPutStringXY(0, 0, "Rx:", -1, TEXT_NORMAL);
    LcdPutStringXY(-1, -1, RxBuffer, RxIndex, TEXT_NORMAL);
    LcdClearToEndOfLine();

    // Display sent characters
    LcdPutStringXY(0, 10, "Tx:", -1, TEXT_NORMAL);
    LcdPutStringXY(-1, -1, TxBuffer, TxIndex, TEXT_NORMAL);
    LcdClearToEndOfLine();

    // Display status message (if any)
    LcdPutStringXY(0, 52, StatusLine, -1, TEXT_NORMAL);
    LcdClearToEndOfLine();

    // Get input from the system and process it.
    RimGetMessage(&msg);

    switch (msg.Device)
    {
      case DEVICE_SYSTEM:
        break;
      case DEVICE_TIMER:
        break;
      case DEVICE_KEYPAD:
        switch (msg.Event)
        {
          case THUMB_ROLL_UP:
            break;
          case THUMB_ROLL_DOWN:
            break;
          case KEY_DOWN:
            ch = (char)msg.SubMsg;
            switch (ch)
            {
              case 'O':
                if (CommOpenPort(9600, 8, COMM_NO_PARITY, 1, 2048, 2048))
                {
                  StatusLine = "Opened 9600 8N1";
                }
                else
                {
                  StatusLine = "Open failed";
                }
                break;

              case 'C':
                CommClosePort();
                StatusLine = "Closed port";
                break;

              case '1':
                StatusLine = "9600 8N1";
                CommSettings(9600, 8, COMM_NO_PARITY, 1);
                break;

              case '2':
                StatusLine = "19200 8N1";
                CommSettings(19200, 8, COMM_NO_PARITY, 1);
                break;

              case '3':
                StatusLine = "9600 7E1";
                CommSettings(9600, 7, COMM_EVEN_PARITY, 1);
                break;

              case '4':
                StatusLine = "19200 7E1";
                CommSettings(19200, 7, COMM_EVEN_PARITY, 1);
                break;

              case 'H':
                StatusLine = "Sent 'Hello'";
                CommSendBuffer("Hello\r", 6);
                break;

              case 'P':  // Pattern
                StatusLine = "Sent 'abcd'";
                CommSendBuffer("abcd", 4);
                break;

              default:
                CommSendChar((BYTE)ch);

                if (ch < 32)
                  ch = '?';

                TxBuffer[TxIndex++] = (char)ch;

                if (TxIndex >= 16)
                {
                  // Scroll the oldest characters out of the tiny buffer
                  // of recently sent characters.
                  memmove(TxBuffer, TxBuffer + 1, 16);
                  TxIndex = 16;
                }

            }
            break;
        }
        break;
      case DEVICE_COM1:
        switch (msg.Event)
        {
          case COMM_RX_AVAILABLE:
            // get the received characters and display them.  Received
            // characters are added to the end of a short string of characters
            // received and displayed on th LCD.

            while ((x = CommReadChar()) != -1)
            {

              if (x < 32)
                x = '?';  // Non printable character.

              // Add the character to the display string.
              RxBuffer[RxIndex++] = (char)x;

              if (RxIndex >= 16)
              {
                // Scroll the oldest characters out of the tiny buffer
                // once it has filled up.
                memmove(RxBuffer, RxBuffer + 1, 16);
                RxIndex = 16;
              }
            }
            break;
          case COMM_CONTROL_CHANGE:
            if (msg.SubMsg)
            {
              StatusLine = "DTR high";
            }
            else
            {
              StatusLine = "DTR low";
            }
            break;
          case COMM_PATTERN_NOTIFY:
            // The notify sequence that was registered by this application
            // has been seen by the OS while monitoring bytes at 9600 8N1 while
            // no app is using the port.

            // When the OS notifies an app that its sequence was see, the app
            // is notified.  The app may then open the com port using whatever
            // settings it intends to use.

            // Note that once an application is using the com port, scanning
            // for notify pattern ceases until the port is closed again.

            if (CommOpenPort(9600, 8, COMM_NO_PARITY, 1, 2048, 2048))
            {
              StatusLine = "Got notify pattern, port opened";
            }
            else
            {
              StatusLine = "Got notify pattern, port open failed";
            }
            break;
        }
        break;
    }
  }
}
