/*-------------------------------------------------------------------------*
 *
 * RIM sample application:
 * REALTIME.C
 *
 * This program demonstrates the use of the real time clock.
 *
 * Copyright 1997-2000 Research In Motion Limited.  All Rights Reserved
 *-------------------------------------------------------------------------*/
#include "Pager.h"

char VersionPtr[] = "Real-Time Clock";
int AppStackSize = 5000;

static char monthname[13][10] = {
  "ERROR  ",
  "January",
  "February",
  "March",
  "April",
  "May",
  "June",
  "July",
  "August",
  "September",
  "October",
  "November",
  "December",
};

//-----------------------------------------------------------------------
// Function to grab the system time and display it on the screen.
//-----------------------------------------------------------------------
void ShowTime(void)
{
  TIME t;
  char DateString[30];

  // Grab the time every time we process a message.
  RimGetDateTime(&t);


  // Format the time into a string
  RimSprintf(DateString, 30, "%4d %s %2d \n" "%02d:%02d", t.year, monthname[t.month], t.date, t.hour, t.minute);

  // Write the time to the display.
  LcdPutStringXY(0, 0, DateString, -1, LCD_MULTILINE);
}

//-----------------------------------------------------------------------
// Application main dispatcher.  Hande a few simple real time operations.
//-----------------------------------------------------------------------
void PagerMain(void)
{
  MESSAGE msg;

  ShowTime();

  for (;;)
  {
    RimGetMessage(&msg);

    // on 'x',set the date to an arbitrary value.
    if (msg.Device == DEVICE_KEYPAD && msg.Event == KEY_DOWN && msg.SubMsg == 'x')
    {
      TIME t;
      RimGetDateTime(&t);
      t.year = 1998;
      t.month = 8;
      t.date = 6;
      RimSetDate(&t);
      LcdClearDisplay();
      ShowTime();
    }

    // on 'z',set the time to an arbitrary value.
    if (msg.Device == DEVICE_KEYPAD && msg.Event == KEY_DOWN && msg.SubMsg == 'z')
    {
      TIME t;
      RimGetDateTime(&t);
      t.hour = 9;
      t.minute = 55;
      t.second = 00;
      RimSetTime(&t);
      LcdClearDisplay();
      ShowTime();
    }

    // on 'a', set the alarm clock one minute into the future.
    if (msg.Device == DEVICE_KEYPAD && msg.Event == KEY_DOWN && msg.SubMsg == 'a')
    {
      TIME t;
      RimGetDateTime(&t);
      t.minute += 1;
      t.second = 00;
      RimSetAlarmClock(&t, TRUE);
    }

    if (msg.Device == DEVICE_RTC)
    {
      // Display the alarm if one is received.
      if (msg.Event == RTC_ALARM_EXPIRED)
      {
        LcdPutStringXY(0, LcdGetFontHeight(FONT_REGULAR) * 2, "Alarm Expired!", -1, 0);
      }

      if (msg.Event == RTC_CLOCK_UPDATE)
      {
        ShowTime();
      }
    }
  }
}
