/*-------------------------------------------------------------------------*
 * RIM sample application:
 * REGMESS.C
 * This program demonstrates the use of registered message callback
 * functions to hande events.
 * Also implements a simple string editing function.
 * Copyright 1997-2000 Research In Motion Limited.  All Rights Reserved
 *-------------------------------------------------------------------------*/
#include <string.h>
#include "Pager.h"

int AppStackSize = 1000;
char VersionPtr[] = "Edit program";

void EditField(int x, int y, char *String, int MaxSize);
int KeyModify(MESSAGE * KeyMsg);
int CounterFunc(MESSAGE * TimerMsg);

char TestString[2000];

// Define an event number for the count timer.  Note that the 16 MSB's
// of a message event are always the device ID, so chossing 0x40001 
// will not be used by other device's events.
#define COUNT_TIMER 0x40001

//-----------------------------------------------------------------------
// Main entry point
//-----------------------------------------------------------------------
void PagerMain(void)
{
  // Register a function to turn a '*' key into a '#' key.
  RimRegisterMessageCallback(KEY_DOWN, 0xffffffff, KeyModify);

  // Set things up for a timer counter and callback to happen
  // independent of calling the edit function.
  RimSetTimer(COUNT_TIMER, 100, TIMER_PERIODIC);
  RimRegisterMessageCallback(COUNT_TIMER, 0xffffffff, CounterFunc);

  EditField(0, 10, TestString, 150);

  // Deregister the key substitution function.
  RimRegisterMessageCallback(KEY_DOWN, 0xffffffff, NULL);
  LcdClearDisplay();

  for (;;)
  {
    RimTaskYield();
    EditField(0, 10, TestString, 150);
    LcdClearDisplay();
  }
}

//-----------------------------------------------------------------------
// Timer callback function.  Displays a count while edit is active.
//-----------------------------------------------------------------------
int CounterFunc(MESSAGE * TimerMsg)
{
  char TempString[20];

#if 0 /* what the hell is this about?! -mid */
  // No need to examine the message - we know it can only be the timer.
  TimerMsg;
#endif
  // Display the seconds since startup.
  RimSprintf(TempString, 20, "Seconds: %d", RimGetTicks() / TICKS_PER_SECOND);
  LcdPutStringXY(50, 0, TempString, -1, 0);
  if(RimGetTicks() / TICKS_PER_SECOND >= 30)
  {
    RimKillTimer(COUNT_TIMER); // Stop the times
    RimRegisterMessageCallback(COUNT_TIMER, 0xffffffff, NULL);
  }
  RimSpeakerBeep(1000,250,25,30);
  return FALSE; // Do not forward the message to RimGetMessage
}

//-----------------------------------------------------------------------
// Registered callback function to modify some key messages.
//-----------------------------------------------------------------------
int KeyModify(MESSAGE * KeyMsg)
{
  if(KeyMsg->SubMsg == '*')
    KeyMsg->SubMsg = '#'; // Turn a '*' into a '#' key.
  return TRUE; // Do forward the message to RimGetMessage
}

//-----------------------------------------------------------------------
// Key fetch routine for edit routine.
//-----------------------------------------------------------------------
#define CURSOR_LEFT
#define CURSOR_RIGHT
int GetKey(void)
{
  MESSAGE msg;
  RimGetMessage(&msg);
  if(msg.Event == KEY_DOWN) return msg.SubMsg;
  if(msg.Event == THUMB_ROLL_UP) return THUMB_ROLL_UP;
  if(msg.Event == THUMB_ROLL_DOWN) return THUMB_ROLL_DOWN;
 return 0; // Other events are ignored
}

//-----------------------------------------------------------------------
// This implements editing of a string
//-----------------------------------------------------------------------
void EditField(int LcdX, int LcdY, char *InputString, int MaxSize)
{
  int c;
  int CursorPos;
  int StringLength;
  int FontHeight;
  int a;
  int lastlines;

  FontHeight = LcdGetFontHeight(-1);
  CursorPos = 0;
  StringLength = 0;
  lastlines = 0;

  for (;;)
  {
    int shown;
    int line;
    int KlugeLen;

    if(StringLength == CursorPos)
    {
      InputString[StringLength] = ' ';
      KlugeLen = StringLength + 1;
    }
    else
      KlugeLen = StringLength;
    shown = 0;
    for (line = 0;; line++)
    {
      int a;
      a = LcdPutStringXY(LcdX,LcdY+line*FontHeight,InputString+shown,KlugeLen-shown,LCD_WORD_WRAP);
      if(a == 0)
        a = LcdPutStringXY(LcdX,LcdY+line*FontHeight,InputString+shown,KlugeLen-shown,0);
      LcdClearToEndOfLine();
      if(CursorPos >= shown && CursorPos < shown + a)
      {
        int CursorX, CursorWidth;  // The cursor is on this line.
        CursorX = LcdGetStringWidth(InputString + shown, -1, CursorPos - shown);
        CursorWidth = LcdGetStringWidth(InputString + CursorPos, -1, 1);
        LcdRasterOp(INVERT_DEST, CursorWidth, FontHeight, NULL, CursorX, LcdY + line * FontHeight, NULL, CursorX, LcdY + line * FontHeight);
      }
      shown += a;
      if(shown >= KlugeLen) break;
    }
    if(lastlines > line)
    {
      LcdSetCursorPoint(LcdX, LcdY + lastlines * FontHeight);
      LcdClearToEndOfLine();
    }
    lastlines = line;
    c = GetKey();
    if(c == 0) continue;
    if(CursorPos > StringLength)
      RimCatastrophicFailure("EDIT BUSTED!");
    switch (c)
    {
      case THUMB_ROLL_DOWN:
        if(CursorPos < StringLength)
          CursorPos += 1;
        break;
      case THUMB_ROLL_UP:
        if(CursorPos > 0) // Use as cursor left
          CursorPos -= 1;
        break;
      case KEY_BACKSPACE:
        if(CursorPos >= 1)
          CursorPos -= 1;
        // NOTE FOLLOWING CHANGE DOES COMPILE !
// __builtin_memcpy(InputString + CursorPos, InputString + CursorPos + 1, StringLength - CursorPos);
        memcpy(InputString + CursorPos, InputString + CursorPos + 1, StringLength - CursorPos);
        if(StringLength >= 1)
          StringLength -= 1;
        break;
      case KEY_ENTER:
        return;
      default:
        if(StringLength < MaxSize)
        {
          for (a = StringLength; a >= CursorPos; a--)
            InputString[a] = InputString[a - 1];
          InputString[CursorPos] = (char)c; // Copy rest of string later...
          CursorPos += 1;
          StringLength += 1;
        }
    }
  }
}

