
/**
 * @author:    Brad Gagne     bgagne@rim.net     July 2000
 * Modified:  Peter Weiler   pweiler@rim.net    Aug 2000
 *
 * SOURCE CODE PROVIDED AS IS. NO WARRANTY OR REPRESENTATION IS MADE BY THE AUTHOR. USE AT YOUR OWN RISK.
 *
 */

#include "pager.h"
#include "Ribbon.h"
#include <stdlib.h>

#include "piano_icon.h"
#include "pianokeys.h"


char VersionPtr[] = "Piano Simulator";

//It is RIM's experience that VC++ 6.0 adds material to the stack
//under debug build mode. Therefore, the stack is increased to avoid
//stack overflows under simulation
#if defined(_DEBUG)
int AppStackSize = 6000;
#else
int AppStackSize = 3000;
#endif //_DEBUG


int volume = 50;		// changed by thumbwheel
char vol_text[4];		

static LcdConfig LcdConf;



// Thread listening for events:
void PianoThread(void) {	

	MESSAGE msg;
	
	PID pid;
    pid.Name = VersionPtr;
    pid.EnableForeground = true;
    pid.Icon = NULL;
    RimSetPID( &pid );

	LcdGetConfig (&LcdConf);
	
	if (LcdConf.width == 160) {
		LcdCopyBitmapToDisplay( &pianokeys_Bitmap, 28, 40 );	
	} else {
		LcdCopyBitmapToDisplay( &pianokeys_Bitmap, 2, 0 );
	}
	// start message loop
	for (;;) {

		if (LcdConf.width == 160) {
			LcdPutStringXY( -1, 10, "Piano Simulator", -1,  TEXT_UNDERLINE | LCD_CENTERED );
			LcdClearToEndOfLine();

			LcdPutStringXY( 28, 120, "Volume: ", -1, TEXT_NORMAL );
			_itoa( volume, vol_text, 10 );
			LcdPutStringXY( -1, -1, vol_text, -1, TEXT_NORMAL );
			LcdClearToEndOfLine();
		} else {
			LcdPutStringXY( 112, 20, "Vol:", -1, TEXT_NORMAL );
			LcdClearToEndOfLine();
			_itoa( volume, vol_text, 10 );
			LcdPutStringXY( 112, 35, vol_text, -1, TEXT_NORMAL );
			LcdClearToEndOfLine();
		}


		RimGetMessage( &msg );	// get a message 

		switch (msg.Device) {

			case DEVICE_KEYPAD:			// if it's a keypad event...

				switch (msg.Event) {

					case THUMB_ROLL_UP:
						if ( volume < 100 ) { volume++; }
						break;

					case THUMB_ROLL_DOWN:
						if ( volume > 0 ) { volume--; }
						break;

					case KEY_DOWN:
					case KEY_REPEAT:
						int x = 1;
						if (msg.Data[0] == SHIFT_STATUS) {
							x = 2;
						}
						if (msg.Data[0] == ALT_STATUS) {
							x = 0.5;
						}

						switch(msg.SubMsg) 
						{

							case KEY_BACKSPACE:
							case 'X':
								RibbonShowRibbon();
								RimTerminateThread();
								break;

							case '*':
							case 'A':
							case 'a':	// B
								RimSpeakerBeep( 493*x, 150, 99, volume );
								break;

							case '/':
							case 'S':
							case 's':	// C
								RimSpeakerBeep( 523*x, 150, 99, volume );
								break;

							case '2':
							case 'W':
							case 'w':	// C#
								RimSpeakerBeep( 554*x, 150, 99, volume );
								break;

							case '+':
							case 'D':
							case 'd':	// D
								RimSpeakerBeep( 587*x, 150, 99, volume );
								break;

							case '3':
							case 'E':
							case 'e':	// D#
								RimSpeakerBeep( 622*x, 150, 99, volume );
								break;

							case '-':
							case 'F':
							case 'f':	// E
								RimSpeakerBeep( 659*x, 150, 99, volume );
								break;

							case '=':
							case 'G':
							case 'g':	// F
								RimSpeakerBeep( 698*x, 150, 99, volume );
								break;

							case '5':
							case 'T':
							case 't':	// F#
								RimSpeakerBeep( 740*x, 150, 99, volume );
								break;

							case ':':
							case 'H':
							case 'h':	// G
								RimSpeakerBeep( 784*x, 150, 99, volume );
								break;

							case '6':
							case 'Y':
							case 'y':	// G#
								RimSpeakerBeep( 831*x, 150, 99, volume );
								break;

							case '\'':
							case 'J':
							case 'j':	// A
								RimSpeakerBeep( 880*x, 150, 99, volume );
								break;

							case '7':
							case 'U':
							case 'u':	// A#
								RimSpeakerBeep( 932*x, 150, 99, volume );
								break;

							case '"':
							case 'K':
							case 'k':	// B
								RimSpeakerBeep( 988*x, 150, 99, volume );
								break;

							case '@':
							case 'L':
							case 'l':	// C
								RimSpeakerBeep( 1047*x, 150, 99, volume );
								break;

							case '0':
							case 'P':
							case 'p':	// C#
								RimSpeakerBeep( 1107*x, 150, 99, volume );
								break;

							case char(13):	// D
								RimSpeakerBeep( 1168*x, 150, 99, volume );
								break;

						// ---------------------------------------------					
						
						}

						break;
				}

		}

	} // message loop

}



void PagerMain( void )
{
    //Register with the ribbon for dynamic thread creation
    //The callback passed is run only when the app is active.
    //Put the icon at position 36 to leave some room for RIM apps
	RibbonRegisterFunction( VersionPtr, &piano_icon_Bitmap, 0, 40, PianoThread, AppStackSize );

}
