////////////////////////////////////////////////////////////////////////////////
// MAIN.C - Minimal monitor for examining system memory locations
////////////////////////////////////////////////////////////////////////////////
#include <pager.h>
#include <ribbon.h>
#include "icon.h"

char VersionPtr[] = "BBmon";
int AppStackSize = 1000;

void BBmonThread(void)
{
  char X,Y;    // Loop variables
  char M=0;    // Display mode toggle
  char NC=0;   // Next char type
  char b[64];  // Formatting String
  char *Addr;  // Tmp addr loop
  char NA[16]; // New address
  char *SA = (char *)0x500000; // Start of RAM 0x500000
  MESSAGE msg;
  PID pid;
  pid.Name = VersionPtr;
  pid.EnableForeground = TRUE;
  pid.Icon = NULL;
  RimSetPID(&pid);
  NA[NC]=0;

  for (;;) // Start message loop
  {
    Addr=SA;
    LcdClearDisplay();
    for(Y=0; Y<19; Y++)
    {
      RimSprintf(b,63, "%08lx:", Addr);  // Convert Address to Hex  
      LcdPutStringXY(0,Y*8,b,-1,0);
      for(X=0; X<8; X++)
      {
        RimSprintf(b,63,M?"%c":"%02x",(unsigned char)*Addr); // Chr or Hex mode
        LcdPutStringXY(48+X*14,Y*8,b,-1,0);
        Addr++;
      }
    }
    LcdDrawLine(DRAW_BLACK,0,152,160,152);   // Draw line
    LcdPutStringXY(0,152,NA,-1,TEXT_INVERT); // New address string

    RimGetMessage(&msg);
    switch (msg.Device)
    {
      case DEVICE_KEYPAD:
        switch (msg.Event)
        {
          case THUMB_ROLL_UP:   // MEMORY DOWN
            if(SA>(char *)0) SA=(char *)((unsigned long)SA)-144;
            break;
          case THUMB_ROLL_DOWN: // MEMORY UP
            if(SA<(char *)0xffffffff) SA=(char *)((unsigned long)SA)+144;
            break;
          case KEY_DOWN:
          {
            switch (msg.SubMsg)
            {
              case KEY_ESCAPE: // ESC key exits
                RibbonShowRibbon();
                RimTerminateThread();
                break;
              case ' ':
                M=(M?0:1); // Mode toggle
                break;
              case KEY_ENTER: // Extract new address from string
                SA=(char *)strtoul(NA,NULL,16);
                NC=0;
                NA[NC]=0;
                break;
              case KEY_BACKSPACE:
                if(NC>0) {NC-=1; NA[NC]=0;}
                break;
              default:
                NA[NC++]=msg.SubMsg;
                NA[NC]=0;
            }
            break;
          }
        }
    }
  } // Message loop
}

void PagerMain(void)
{
  RibbonRegisterFunction(VersionPtr,&IconBMP,0,40,BBmonThread,AppStackSize);
}
