/*-------------------------------------------------------------------------*
 * RIM sample application:
 * EDITFUNC.C
 * Shows how to edit some text in a relatively simple manner.
 * Copyright 1997-2000 Research In Motion Limited.  All Rights Reserved
 *-------------------------------------------------------------------------*/
#include <pager.h>
#include "editfunc.h"
//#include <memory.h>
//#include <string.h>

#define MAX_LINES 100
#define FONTHEIGHT LcdGetFontHeight(-1)
#define SHOW_LINES 6

// These variables contain the state of the edit function.
struct {
  char *InputString;
  int CursorXCenter;
  int CursorStream;
  int FirstLineShown;
  int Flags;
  int MaxLength;
  int LcdX;
  int LcdY;
} State;

// These variables contain computed statistics that can be reconstructed.
static struct {
  int CursorLine;
  int CurosrChar;
  int Length;
  int CursorXShown;
  int CursorWidth;

  int FirstIndexedLine;
  int FirstUnindexed;

  short LineStarts[MAX_LINES + 1];
} Computed;

static LcdConfig LcdConf;

//-----------------------------------------------------------------------------
// Word-wrap index the text.
//-----------------------------------------------------------------------------
static void ReindexInput(int IndexToLine)
{
  int StringIndex;
  int line;

  if (Computed.FirstUnindexed < 0)
    Computed.FirstUnindexed = 0;

  //RimDebugPrintf("Index %d to %d",Computed.FirstUnindexed,IndexToLine);

  StringIndex = Computed.LineStarts[Computed.FirstUnindexed];
  line = Computed.FirstUnindexed;

  for (;;)
  {
    int LastWordEnd;
    int LLen;
    int c;

    if (line > IndexToLine)
      break;

    LastWordEnd = 0;
    LLen = 0;
    for (;;)
    {
      c = State.InputString[StringIndex];

      StringIndex++;

      LLen += LcdGetStringWidth((char *)&c, -1, 1);

      if (c == '\n' || c == '\r' || c == 0)
      {
        break;
      }

      if (c == ' ' || c == '\t')
      {
        // Non printing character.  Note it as a safe cutoff point.
        LastWordEnd = StringIndex;
      }
      else
      {
        if ((unsigned)LLen > LcdConf.width)
        {
          if (LastWordEnd)
          {
            StringIndex = LastWordEnd;
          }
          else
          {
            StringIndex -= 1;
          }
          break;
        }
      }
    }

    line += 1;

    Computed.LineStarts[line] = (short)StringIndex;

    if (c == 0)
    {
      break;
    }
  }
  Computed.FirstUnindexed = line;
}

//-----------------------------------------------------------------------------
// Determine wether indexing is necessary.
//-----------------------------------------------------------------------------
static int LineStartIndex(int LineNum)
{

  if (LineNum < 0)
    return 0;

  if (LineNum < Computed.FirstIndexedLine)
  {
    // Can't index backwards.  Reindex from the top.
    Computed.FirstIndexedLine = 0;
    Computed.FirstUnindexed = 0;
  }

  if (LineNum > Computed.FirstUnindexed)
  {
    // Index forward to the desired line.
    ReindexInput(LineNum);
  }

  return Computed.LineStarts[LineNum];
}

//-----------------------------------------------------------------------------
// Given cursor X and Y position, find the stream position.
//-----------------------------------------------------------------------------
static void SetStreamFromXY(void)
{
  // Set the stream position based on CursorXShown and Computed.CurosrLine;
  int ScanPoint;
  int LLen;

  LLen = 0;

  if (Computed.CursorLine < 0)
  {
    Computed.CursorLine = 0;
  }

  ScanPoint = LineStartIndex(Computed.CursorLine);

  if (ScanPoint >= Computed.Length)
  {
    Computed.CursorLine -= 1;
    ScanPoint = LineStartIndex(Computed.CursorLine);
  }

  for (;;)
  {
    LLen += LcdGetStringWidth(State.InputString + ScanPoint, -1, 1);

    if (LLen >= State.CursorXCenter + 1)
    {
      break;
    }

    if (ScanPoint + 1 >= LineStartIndex(Computed.CursorLine + 1))
    {
      break;
    }
    ScanPoint += 1;
  }
  State.CursorStream = ScanPoint;
}
//-----------------------------------------------------------------------------
// Given cursor stream position, find its x and y position.
//-----------------------------------------------------------------------------
static void SetCursorXYFromStream(void)
{
  int line;
  line = Computed.CursorLine;

  if (State.CursorStream < 0)
    State.CursorStream = 0;
  if (State.CursorStream > Computed.Length)
    State.CursorStream = Computed.Length;

  for (;;)
  {
    if (LineStartIndex(line) <= State.CursorStream)
    {
      if (LineStartIndex(line + 1) > State.CursorStream)
      {
        // Found the line with the cursor on it.
        Computed.CursorLine = line;
        Computed.CursorWidth = LcdGetStringWidth(State.InputString + State.CursorStream, -1, 1);
        Computed.CursorXShown = LcdGetStringWidth(State.InputString + LineStartIndex(line), -1, State.CursorStream - LineStartIndex(line));

        if (Computed.CursorXShown > LcdConf.width - 2)
        {
          // This can happen if the line ends with lots of spaces
          Computed.CursorXShown = LcdConf.width - 2;
        }
        if (Computed.CursorWidth < 2)
          Computed.CursorWidth = 2;

        break;
      }
      else
      {
        line += 1;
        // Later: Check if its beyond the indexed region or past the end of text.
      }
    }
    else
    {
      line -= 1;
      // Later: Check if its beyond the indexed region.
      if (line < 0)
      {
        // Screwed up.
        RimCatastrophicFailure("Edit broken");
      }
    }
  }

}

//-----------------------------------------------------------------------------
// Show the word wrapped text on the screen.
//-----------------------------------------------------------------------------
void RedrawText(void)
{
  int a;

  if (Computed.CursorLine < State.FirstLineShown)
  {
    State.FirstLineShown = Computed.CursorLine;
  }
  else if (Computed.CursorLine >= State.FirstLineShown + SHOW_LINES)
  {
    State.FirstLineShown = Computed.CursorLine - SHOW_LINES + 1;
  }


  // Display the text with the cursor!
  for (a = 0; a < SHOW_LINES; a++)
  {
    int Li;
    int LineChars;
    int LineStart;

    Li = a + State.FirstLineShown;

    LineStart = LineStartIndex(Li);

    // Check if the line is past the end.
    if (LineStart < Computed.Length || Computed.Length == 0)
    {

      // Clear out the line.
      LcdRasterOp(WHITE, LcdConf.width, FONTHEIGHT, NULL, 0, 0, NULL, State.LcdX, State.LcdY + a * FONTHEIGHT);

      // Show the line.
      LineChars = LineStartIndex(Li + 1) - LineStart;

      LcdPutStringXY(State.LcdX, State.LcdY + a * FONTHEIGHT, State.InputString + LineStart, LineChars, 0);
    }

    if (Li == Computed.CursorLine)
    {
      // Show the cursor.
      LcdRasterOp(INVERT_DEST, Computed.CursorWidth, FONTHEIGHT, NULL, 0, 0, NULL, State.LcdX + Computed.CursorXShown, State.LcdY + a * FONTHEIGHT);
    }

    if (LineStart >= Computed.Length)
      break;
  }
}

//-----------------------------------------------------------------------------
// Start an edit function.
//-----------------------------------------------------------------------------
void EditBegin(int UseX, int UseY, char *StringPassed, int MaxLength, DWORD Flags)
{
  LcdGetConfig(&LcdConf);

  State.LcdX = UseX;
  State.LcdY = UseY;

  State.Flags = Flags;

  State.FirstLineShown = 0;
  State.CursorStream = (Flags & EDIT_START_AT_EOL) ? strlen(StringPassed) : 0;
  State.MaxLength = MaxLength;
  State.InputString = StringPassed;
  State.CursorXCenter = 0;

  EditModified();
}

//-----------------------------------------------------------------------------
// Indicate that edit has been modified.
//-----------------------------------------------------------------------------
void EditModified(void)
{
  Computed.FirstIndexedLine = 0;
  Computed.FirstUnindexed = 0;
  Computed.LineStarts[0] = 0;
  Computed.Length = strlen(State.InputString);

  SetCursorXYFromStream();

  RedrawText();
}

//-----------------------------------------------------------------------------
// Process edit input
//-----------------------------------------------------------------------------
int EditProcessMessage(MESSAGE * msg)
{
  int ret = TRUE;
  int a;

  switch (msg->Event)
  {
    case THUMB_ROLL_DOWN:
      if (msg->Data[0] & ALT_STATUS)
      {
        State.CursorStream += 1;
      }
      else
      {
        Computed.CursorLine += 1;
        SetStreamFromXY();
        goto keep_x;
      }
      break;

    case THUMB_ROLL_UP:
      if (msg->Data[0] & ALT_STATUS)
      {
        // Means cursor left.
        State.CursorStream -= 1;
      }
      else
      {
        Computed.CursorLine -= 1;
        SetStreamFromXY();
        goto keep_x;
      }
      break;

    case KEY_DOWN:
    case KEY_REPEAT:
      // Add a keystroke.  May affect the line
      // before the cursor line.
      Computed.FirstUnindexed = Computed.CursorLine - 1;

      switch (msg->SubMsg)
      {
        case KEY_BACKSPACE:
          if (State.CursorStream > 0)
          {
            State.CursorStream -= 1;
          }
          else
          {
            break;
          }

        case KEY_DELETE:
          memcpy(State.InputString + State.CursorStream, State.InputString + State.CursorStream + 1, Computed.Length - State.CursorStream);
          Computed.Length -= 1;
          break;

        case KEY_ENTER:
          if (State.Flags & EDIT_IGNORE_CR)
          {
            ret = FALSE;
            goto keep_x;
          }
          break;

        default:
          if (State.Flags & (EDIT_DEC_NUMERIC | EDIT_HEX_NUMERIC))
          {
            if (msg->SubMsg >= 'a' && msg->SubMsg <= 'z')
            {
              // convert to lower case.
              msg->SubMsg = msg->SubMsg + 'A' - 'a';
            }
            // Check if it is usable as a number character

            if (msg->SubMsg == 'E' && (State.Flags & EDIT_HEX_NUMERIC))
            {
              // E is both a number and hexadecimal.  Do not substiture
              // number in hex entry mode.
              goto default_char;
            }

            for (a = 0; a < 10; a++)
            {
              if (msg->SubMsg == (unsigned)"PQWERTYUIO"[a])
              {
                msg->SubMsg = '0' + a;
                goto default_char;
              }
            }
          }

        default_char:
          if ((State.Flags & EDIT_DEC_NUMERIC) && (unsigned)(msg->SubMsg - '0') > 9)
          {
            break;
          }
          else if (State.Flags & EDIT_HEX_NUMERIC)
          {
            if ((unsigned)(msg->SubMsg - '0') > 9 && (unsigned)((msg->SubMsg & ~0x20) - 'A') > 5)
            {
              break;
            }
          }

          if (Computed.Length < State.MaxLength)
          {
            Computed.Length += 1;
            for (a = Computed.Length; a >= State.CursorStream; a--)
            {
              State.InputString[a] = State.InputString[a - 1];
            }
            State.InputString[State.CursorStream] = (char)msg->SubMsg;
            State.CursorStream += 1;
          }
          break;
      }
      break;

    default:
      ret = FALSE;
      goto keep_x;

  }

  SetCursorXYFromStream();
  State.CursorXCenter = Computed.CursorXShown + (Computed.CursorWidth / 2);
keep_x:;

  SetCursorXYFromStream();
  RedrawText();
  return (ret);
}

//-----------------------------------------------------------------------------
// End an edit function.
//-----------------------------------------------------------------------------
void EditEnd(void)
{
  int a = Computed.CursorLine - State.FirstLineShown;
  LcdRasterOp(INVERT_DEST, Computed.CursorWidth, FONTHEIGHT, NULL, 0, 0, NULL, State.LcdX + Computed.CursorXShown, State.LcdY + a * FONTHEIGHT);
}
