/*-------------------------------------------------------------------------*
 * RIM sample application:
 * FILEDEMO.C
 * This app demosntrates how to do some simple file system and
 * user interface manipulation.
 * Note that more sophisitcated API's exist as part of the RIM applications
 * DLL's.  However, this app just uses the OS's API straight.
 * Copyright 1997-2000 Research In Motion Limited.  All Rights Reserved
 * CONVERTED RIBBON VERSION
 *-------------------------------------------------------------------------*/
#include <pager.h>
#include <ribbon.h>
#include "icon.h"

#include <string.h>

#include "popupmenu.h" // Functions for sample module POPUPMENU.C
#include "editfunc.h"  // Functions for sample module EDITFUNC.C

char VersionPtr[] = "File demo";
int AppStackSize = 2000;

#define MAX_ITEMS 100

// Table of handle numbers for quickly accessing pieces of the database.
static short HandleTable[MAX_ITEMS];

// Address of the database pointer table.
static void **PointerTable;

// Number of items in the database.
static int NumItems;

// Handle to our database.
short DatabaseHandle;
static LcdConfig LcdConf;

//-----------------------------------------------------------------------------
// This function reads all the database handles into an array.
//-----------------------------------------------------------------------------
void GetDatabaseHandles(void)
{
  short CurrentHandle;
  // Get the handle to the first record in the database.
  CurrentHandle = DbFirstRec(DatabaseHandle);
  // Only read 100 handles out (that's all this app has the array declared to)
  for (NumItems = 0; NumItems < 100; NumItems++)
  {
    // Check if the end of the database is reached.
    if (CurrentHandle < 0) break;
    HandleTable[NumItems] = CurrentHandle; // Store the handle for our list
    CurrentHandle = DbNextRec(CurrentHandle); // Grab the next record
  }
}

//-----------------------------------------------------------------------------
// Edit or add a record.
//-----------------------------------------------------------------------------
void DoEditScreen(short HandleNo)
{
  static char EditBuffer[500];
  MESSAGE msg;
  LcdClearDisplay();
  memset(EditBuffer, 0, sizeof(EditBuffer)); // Clear the edit buffer
  if (HandleNo >= 0)
  {
    int Size;
    Size = DbRecSize(HandleNo, TRUE); // Get the size of the record
    // Copy the record out of flash into RAM.  Note that flash is memory
    // mapped, but the following donditions apply:
    // * The record in flash canot be written to directly.
    // * The pointer in the pointer table may change on file system writes
    //   by this or other apps when this one yields control
    // As such, copying the data to RAM is the simplest way to go when
    // processing just one record at a time.
    memcpy(EditBuffer, PointerTable[HandleNo], Size);
  }
  // EditBegin is a function implemented in source file SIMPLEEDIT.C
  // This function handles proportinally spaced editing of text.
  // EditBegin is part of this sample app, not a UI32 function.
  EditBegin(0, 10, EditBuffer, 500, 0);
  for (;;)
  {
    // redraw all of the screen with every message.  
    // This is wasteful, but on such a small screen, the delay is not noticeable.
    if (HandleNo >= 0)
    {
      LcdPutStringXY(0, 0, "Edit record text", -1, 0);
    }
    else
    {
      LcdPutStringXY(0, 0, "Enter new record text", -1, 0);
    }
    LcdDrawLine(DRAW_BLACK, 0, 9, LcdConf.width, 9);
    RimGetMessage(&msg);
    // Any non user interface messages we might catch can be processed
    // here.  This app doesn't care about other input.
    if (msg.Event == THUMB_CLICK)
    {
      switch (PopupMenuWaitForSelection("Hide menu|Save Record;Cancel Edit", 1))
      {
        case 0:  // Hide menu.
          break;
        case 1:  // Save record
          if (HandleNo >= 0)
          {
            if (DbReplaceRec(HandleNo, strlen(EditBuffer) + 1, EditBuffer) < 0)
            {
              // If this fails, probably ran out of space, or a bug in this
              // program.  this program just gives up when either happens.
              RimCatastrophicFailure("Replace failed!");
            }
          }
          else
          {
            if (DbAddRec(DatabaseHandle, strlen(EditBuffer) + 1, EditBuffer) < 0)
            {
              // If this fails, probably ran out of space.  This program
              // just gives up when that happens.
              RimCatastrophicFailure("Save failed");
            }
          }
          return;
        case 2:  // Cancel edit
          return; // Don't save it.  Just return
      }
      // Clear the display to get rid of the menu.
      // Gets redrawin by EditProcessMessage
      LcdClearDisplay();
    }
    // Pass the message to the edit screen code.
    EditProcessMessage(&msg);
  }
}

//-----------------------------------------------------------------------------
// Main pager function.
//-----------------------------------------------------------------------------
void FdemoThread(void)
{
  MESSAGE msg;
  int TopmostIndex;
  int Selection;
  int a;
  PID pid;
  pid.Name = VersionPtr;
  pid.EnableForeground = TRUE;
  pid.Icon = NULL;
  RimSetPID(&pid);
  RimRequestForeground(RimGetCurrentTaskID());

  LcdGetConfig(&LcdConf);
  // Get the handle to the database.  If none exists, one is created by that name.
  DatabaseHandle = DbGetHandle("Sample Database");
  // Get the handles to all the database entries in the database.
  GetDatabaseHandles();
  // Get the address of the pointer table used for dereferencing the handles.
  // DbPointTable returns a pointer to a table of pointers.
  PointerTable = DbPointTable();
  // Note that while the values (addresses) stored in the array whose address is
  // returned by DbPointTable() may change, the actual address of the pointer table
  // never moves.  Thus, the program only needs to call DbPointTable() once, but
  // needs todereference using the pointer table after every Yield.

  TopmostIndex = 0;
  Selection = 0;

  for (;;)
  {
    LcdClearDisplay();
    // Draw the title of the screen.
    LcdPutStringXY(0, 0, "Data entries:", -1, 0);
    LcdDrawLine(DRAW_BLACK, 0, 9, LcdConf.width, 9);
    // Ensure selection is within available items.
    if (Selection >= NumItems)
      Selection = NumItems - 1;
    if (Selection < 0)
      Selection = 0;
    // Scroll the display if need be.
    if (Selection < TopmostIndex)
      TopmostIndex = Selection;
    if (Selection > TopmostIndex + 5)
      TopmostIndex = Selection - 5;
    if (NumItems > 0)
    {
      int y;
      for (a = 0; a < 6; a++) // Display 6 items on the screen
      {
        int RecNum;
        RecNum = a + TopmostIndex;
        if (RecNum >= NumItems)
          break;
        // All of our records are just strings.  Note that using the PointerTable,
        // we can call LcdPutStringXY with a pointer directly into flash.
        // Caution must be taken here.  The pointer is only good until the program
        // yields control to other apps or writes the the file system.  As
        // LcdPutStringXY does neither of these, it is safe to use the pointer here.
        LcdPutStringXY(0, 10 + a * 9, PointerTable[HandleTable[RecNum]], -1, LCD_ELLIPSIS);
      }
      // Highlight the current selection.
      y = (Selection - TopmostIndex) * 9 + 10;
      LcdRasterOp(INVERT_DEST, LcdConf.width, 9, NULL, 0, y, NULL, 0, y);
    }
    else
    {
      LcdPutStringXY(0, 15, "Click roller to start", -1, 0);
    }

    RimGetMessage(&msg); // Get user input

    if(msg.Device==DEVICE_KEYPAD&&msg.Event==KEY_DOWN)
    {
      if(msg.SubMsg==KEY_ESCAPE) // ESC quits
      {
        RibbonShowRibbon();
        RimTerminateThread();
      }
    }

    switch (msg.Event)
    {
      case THUMB_ROLL_DOWN:
        Selection += 1;
        break;
      case THUMB_ROLL_UP:
        Selection -= 1;
        break;
      case THUMB_CLICK:
        switch (PopupMenuWaitForSelection("Hide Menu|Edit;Add;Delete|Switch apps|Cancel", 2))
        {
          case 0:  // Hide menu.
            break;
          case 1:  // Edit
            if (NumItems == 0)
            {
              goto do_add;
            }
            DoEditScreen(HandleTable[Selection]);
            break;
          case 2:  // Add a new item.
do_add:
            DoEditScreen(-1);
            // After edit, a new item may have been added. Re-grab all
            // the handles.
            GetDatabaseHandles();
            break;
          case 3:  // Delete the record from this database.
            DbDeleteRec(DatabaseHandle, HandleTable[Selection]);
            // The record is now deleted.  Re-grabbing all the handles
            // is an inefficient but simple way of getting our handle table
            // up to date.
            GetDatabaseHandles();
            break;
          case 4:
            // Go to the os's task switcher
            RimInvokeTaskSwitcher();
            break;
        }
      // Clear the display to get rid of the menu.
      // Gets redrawin with next processing of a message.
        LcdClearDisplay();
    }
  }
}

void PagerMain(void)
{
  RibbonRegisterFunction(VersionPtr, &IconBMP, 0, 40, FdemoThread, AppStackSize);
}
